----------------------------------------------------------------------
-- schnelle 10 Bit PWM 
-- 27.07.2008
--
-- PWM-Wert 0..1023 wird seriell geladen, bernahme des neuen Datenwortes
-- mit fallender Flanke von Load
--
-- fr XC9536
----------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity PWM is
    Port ( ShiftClk : in std_logic;		  -- Takt fr Eingangs-Schieberegister (= PWM-Wert)
           SerData : in std_logic;		  -- Serielle Daten fr Schieberegister
           Load : in std_logic;			  -- Daten aus Schieberegister bei fallender Flanke bernehmen
			  PWMClk : in std_logic;		  -- Takt fr PWM
           PWMOut : out std_logic);		  -- PWM-Ausgang
end PWM;

architecture Behavioral of PWM is

signal SchiebeRegister : std_logic_vector(9 downto 0);
signal PWMWert : std_logic_vector(9 downto 0);
signal Counter : integer range 0 to 1023 :=0;
signal LoadMerker : std_logic;
begin

process (ShiftClk)
begin

	if rising_edge (ShiftClk) then
		SchiebeRegister(9 downto 0) <= SchiebeRegister(8 downto 0) & SerData;
	end if;

end process;

process (PWMClk)

begin
	if rising_edge (PWMClk) then

		Counter <= Counter +1;

		if Load = '1' then
			LoadMerker <= '1';
		elsif	LoadMerker = '1' and Counter = 0 then	  -- Neuen Wert erst laden wenn Counter = 0
			LoadMerker <= '0';
			PWMWert <= Schieberegister;
		end if;

		if Counter < PWMWert then
			PWMOut <= '1';
		else
			PWMOut <= '0';
		end if;
	end if;
end process;	

end Behavioral;
